#!/usr/bin/env python

import os, sys

from distutils.core import setup, Extension

try:
    SAGE_LOCAL = os.environ['SAGE_LOCAL']
except:
    if sys.argv[1] != "install":
        SAGE_LOCAL = ""
    else:
        print "The environment variable SAGE_LOCAL must be set"
        print "to the local build prefix."
        sys.exit(1)

pyrex_compile_options = ['-Wno-unused-label', '-Wno-unused-function',
                         '-Wno-parentheses',  '-Wno-unused-variable',
                         '-I%s/include'%SAGE_LOCAL]

pyrex_link_options = ['-L%s/lib'%SAGE_LOCAL]

setup(name='sage',
      version='0.3',
      description='SAGE: System for Algebra and Geometry Experimentation',
      author='William Stein',
      author_email='was@math.harvard.edu',
      url='http://modular.fas.harvard.edu/SAGE',
      packages = ['sage',
                  'sage.algebras',
                  'sage.ellcurve',
                  'sage.functions',
                  'sage.groups',
                  'sage.interface',
                  'sage.linalg',
                  'sage.misc',
                  'sage.modular',
                  'sage.modular.abvar',
                  'sage.modular.modform',
                  'sage.modular.modsym',
                  'sage.modular.ssmod',
                  'sage.pari',
                  'sage.plot',
                  'sage.rings',
                  'sage.tables',
                  'sage.structure'],
      
      ext_modules = [\

         # pari
         Extension('sage.pari.py_pari',
                   ['sage/pari/py_pari.c'],
                   libraries=['pari'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         # rings
         Extension('sage.rings.arith_c',
                   ['sage/rings/arith_c.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         Extension('sage.rings.arith_gmp',
                   ['sage/rings/arith_gmp.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.rings.element',
                   ['sage/rings/element.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.rings.integer',
                   ['sage/rings/arith_c.c',
                    'sage/rings/integer.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.rings.intmod_pyx',
                   ['sage/rings/intmod_pyx.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.rings.polynomial_pyx',
                   ['sage/rings/polynomial_pyx.c',
                    'sage/rings/arith_gmp.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         Extension('sage.rings.rational',
                   ['sage/rings/rational.c',
                    'sage/rings/arith_c.c', \
                    'sage/rings/integer.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.rings.sparse_poly',
                   ['sage/rings/sparse_poly.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         # linalg
         Extension('sage.linalg.sparse_matrix_pyx',
                   ['sage/linalg/sparse_matrix_pyx.c',
                    'sage/rings/integer.c',
                    'sage/rings/rational.c',
                    'sage/rings/arith_c.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         Extension('sage.linalg.dense_matrix_pyx',
                   ['sage/linalg/dense_matrix_pyx.c',
                    'sage/rings/integer.c',
                    'sage/rings/rational.c',
                    'sage/rings/arith_c.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         # misc
         Extension('sage.misc.search',
                   ['sage/misc/search.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         # modsym
         Extension('sage.modular.modsym.heilbronn',
                   ['sage/modular/modsym/heilbronn.c',
                    'sage/modular/modsym/p1list.c',
                    'sage/rings/arith_c.c'],
                   libraries = ['gmp', 'm'], 
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         Extension('sage.modular.modsym.p1list',
                   ['sage/modular/modsym/p1list.c',
                    'sage/rings/arith_c.c'],
                   libraries = ['gmp'], 
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options)
         ]
     )
