//MRANK12.H -- definition of base class rank12, common base for separate 
//             classes rank1 and rank2 for computing rank via general 
//             2-descent and descent via 2-isogeny repectively
//
#include "compproc.h"
#include "points.h"
#include "mwprocs.h"
#include "mquartic.h"

class rank1;
class rank2;

class rank12 {
protected: 
  Curvedata* the_curve;
  int verbose, certain, success, selmer_only, do_second_descent;
  long num_aux;
  long rank, selmer_rank;
  Curvedata IJ_curve;  // [0,0,0,-27*I,-27*J]
  bigint tr_u,tr_r,tr_s,tr_t;  // transformation from latter to minimal curve
  long lim1, lim2;
public:

// Constructor:
//
// sel is selmer_only switch
// firstlim is bound on |x|+|z|
// secondlim is bound on log max {|x|,|z| }, i.e. logarithmic
// n_aux only relevant for general 2-descent when 2-torsion trivial
// n_aux=-1 causes default to be used (depends on method)
// second_descent only relevant for descent via 2-isogeny

  rank12(){;}
  rank12(Curvedata* ec, 
         int verb=0, int sel=0, 
         long firstlim=20, long secondlim=5, 
	 long n_aux=-1, int second_descent=1)
    :the_curve(ec), verbose(verb), selmer_only(sel), 
    lim1(firstlim), lim2(secondlim), num_aux(n_aux), 
    do_second_descent(second_descent) {;}
  ~rank12() {;}
  long getrank() const {return rank;}
  long getselmer() const {return selmer_rank;}
  long getcertain()  const {return certain;}
  int ok()      const {return success;}
  virtual void listpoints()=0;
  virtual void listpoints(Curvedata* CD_orig, 
			  const bigint& u, const bigint& r, 
			  const bigint& s, const bigint& t)=0;
  virtual vector<Point> getgens() const =0;
  virtual vector<Point> getpoints() =0;
};


class two_descent {
private: rank12 * r12;  // does all the work
  int verbose, two_torsion_exists, selmer_only;
  int success, certain, fullmw;
  long rank, selmer_rank, sat_bound;
  mw mwbasis;
public:
// Constructor:
//
// sel is selmer_only switch
// firstlim is bound on |x|+|z|
// secondlim is bound on log max {|x|,|z| }, i.e. logarithmic
// n_aux only relevant for general 2-descent when 2-torsion trivial
// n_aux=-1 causes default to be used (depends on method)
// second_descent only relevant for descent via 2-isogeny

  two_descent(Curvedata* ec, 
	      int verb=0, int sel=0, 
	      long firstlim=20, long secondlim=5, 
	      long n_aux=-1, int second_descent=1);
  long getrank() const {return rank;}
  long getselmer() const {return selmer_rank;}
  long getcertain()  const {return certain;}
  int ok()      const {return success;}
  int get2t() const {return two_torsion_exists;}
  int getfullmw() const {return fullmw;}
  bigfloat regulator() {return mwbasis.regulator();}
  vector<Point> getbasis() {return mwbasis.getbasis();}
  vector<Point> getbasis(Curvedata* CD_orig, 
			  const bigint& u, const bigint& r, 
			  const bigint& s, const bigint& t);
  void report_rank() const;
  void saturate(long sat_bd); // =0 for none
  void show_gens(int change,
		 Curvedata* CD_orig, 
		 const bigint& u, const bigint& r, 
		 const bigint& s, const bigint& t);
  void show_result_status();
  void pari_output(int change,
		   Curvedata* CD_orig, 
		   const bigint& u, const bigint& r, 
		   const bigint& s, const bigint& t);
};

