// egr.h : functions for testing good reduction of points, component
// groups etc

int HasGoodReduction(const Point& P, const bigint& p);
int HasGoodReduction(const Point& P, const vector<bigint>& plist, bigint& p0);
int OrderInComponentGroup(const Point& P, const bigint& p, int group_order=0);
int InSameComponent(const Point& P, const Point& Q, const bigint& p);

// replaces the (independent) points with a new set which spans the
// subgroup of the original with good reduction at p, returning the
// index
int gr1prime(vector<Point>& Plist, const CurveRed& E, const bigint& p);

// replaces the (independent) points with a new set which spans the
// subgroup of the original with good reduction at all p in plist,
// returning the overall index
int grprimes(vector<Point>& Plist, const CurveRed& E, const vector<bigint>& plist);

// replaces the (independent) points with a new set which spans the
// subgroup of the original with good reduction at all p,
// returning the overall index
int egr_subgroup(vector<Point>& Plist, int real_too=1);




// Returns [m] for cyclic of order m, [2,2] for 2*2 (type I*m, m even only)
vector<int> ComponentGroup(const CurveRed& E, const bigint& p);



// Given a list of points P1,...,Pn and a prime p, this returns a
// vector [c1,c2,...,cn] where the image of Pi in the component group
// is ci mod m, where m is the exponent of the component group at p.
// 
// Each ci is a vector of length 1 or 2 (the latter for when the
// component group is C2xC2), not just an integer.
//
// If p=0 then m=1 or 2 (m=2 iff there are two real components and at
// least one P_i is not in the connected component)
//

vector<vector<int> >  MapPointsToComponentGroup(const vector<Point>& Plist, 
						const bigint& p);

