// elliptic logarithm functions

#ifndef _ELOG_H_
#define _ELOG_H_

#include "curve.h"
#include "points.h"
#include "cperiods.h"

// Given an elliptic curve and its (precomputed) periods, and a point
// P=(x,y), returns the unique complex number z such that 

// (1) \wp(z)=x+b2/12, \wp'(z)=2y+a1*x+a3, 

// (2) either z is real and 0\le z\lt w1, or Delta>0, z-w2/2 is real
// and 0\le z-w2/2\le w1.  

// Here, [w1,w2] is the standard period lattice basis

// c.f. Cohen page 399

bigcomplex elliptic_logarithm(const Curvedata& E, const Cperiods& per, const Point& P);

// Cperiods is a class containing a basis for the period lattice L;
// it knows how to compute points from z mod L; so this function
// effectively does the same as PARI's ellztopoint()
//
// User supplies a denominator for the point; if it doesn't work, the
// Point returned is 0 on the curve
//

Point ellztopoint(Curvedata& E,  Cperiods& per, const bigcomplex& z, const bigint& den);

// Returns a (possibly empty) vector of solutions to m*Q=P

// First version will compute the Cperiods itself, so best to use the
// second one if more than one call is to be made for the same curve

vector<Point> division_points(Curvedata& E,  const Point& P, int m);
vector<Point> division_points(Curvedata& E,  Cperiods& per, const Point& P, int m);

// Returns a vector of solutions to m*Q=0 (including Q=0)

// First version will compute the Cperiods itself, so best to use the
// second one if more than one call is to be made for the same curve

vector<Point> torsion_points(Curvedata& E,int m);
vector<Point> torsion_points(Curvedata& E,  Cperiods& per, int m);

#endif
