// Integral LLL algorithm (Cohen's book page 94)
// 
// b is an array of n+1 vectors indexed from 0 to n.  
// b[1]...b[n] are the lattice basis, while b[0] holds the coefficients 
// of the (diagonal) Gram matrix, so the inner product of b[i] and b[j] 
// is sum(k,b[0][k]b[i][k]*b[j][k]). 
//

#ifndef _ILLL_H
#define _ILLL_H      1
                           //flags that this file has been included


bigint sdot(const vec_m* b, int i, int j);

void lll_reduce(const int n, vec_m* b);

//
// Uses Pohst-Zassenhaus Algorithm (page 190) to find all vectors of
// length < c, where the quadratic form is again given by b[0].
//
// NB The following DOES NOT WORK: I blindly implemented P-Z without
// doing the necessary preliminary completing of the square.
// So DO NOT USE
//
//void list_short_vecs(const int n, vec_m* b, const bigint& c);

#endif
